@@ -12,7 +12,6 @@ from deprecated.sphinx import versionadded
 from packaging.version import Version
 
 from coredis._utils import b, clusterdown_wrapper, first_key, nativestr
-from coredis.cache import AbstractCache
 from coredis.commands._key_spec import KeySpec
 from coredis.commands.constants import CommandName
 from coredis.commands.core import CoreCommands
@@ -27,7 +26,6 @@ from coredis.exceptions import (
     BusyLoadingError,
     ClusterDownError,
     ClusterError,
-    ClusterRoutingError,
     ConnectionError,
     MovedError,
     RedisClusterException,
@@ -126,21 +124,19 @@ class ClusterMeta(ABCMeta):
                 if (wrapped := add_runtime_checks(method)) != method:
                     setattr(kls, name, wrapped)
                     method = wrapped
-            if doc_addition and not hasattr(method, "__cluster_docs"):
+            if doc_addition:
 
                 def __w(func: Callable[P, Awaitable[R]]) -> Callable[P, Awaitable[R]]:
                     @functools.wraps(func)
                     async def _w(*a: P.args, **k: P.kwargs) -> R:
                         return await func(*a, **k)
 
-                    _w.__doc__ = f"""{textwrap.dedent(method.__doc__ or "")}
+                    _w.__doc__ = f"""{textwrap.dedent(method.__doc__)}
 {doc_addition}
                 """
                     return _w
 
-                wrapped = __w(method)
-                setattr(wrapped, "__cluster_docs", doc_addition)
-                setattr(kls, name, wrapped)
+                setattr(kls, name, __w(method))
         return kls
 
 
@@ -158,9 +154,6 @@ class RedisMeta(ABCMeta):
         return kls
 
 
-RedisConnectionT = TypeVar("RedisConnectionT", bound="RedisConnection")
-
-
 class RedisConnection:
     encoding: str
     decode_responses: bool
@@ -266,7 +259,7 @@ class RedisConnection:
             decode=decode if decode is None else bool(decode)
         )
 
-    async def initialize(self: RedisConnectionT) -> RedisConnectionT:
+    async def initialize(self) -> RedisConnection:
         await self.connection_pool.initialize()
         return self
 
@@ -299,8 +292,6 @@ class AbstractRedis(
     """
 
     server_version: Optional[Version]
-    protocol_version: Literal[2, 3]
-    cache: Optional[AbstractCache]
 
     async def scan_iter(
         self,
@@ -466,7 +457,6 @@ class Redis(
         client_name: Optional[str] = ...,
         protocol_version: Literal[2, 3] = ...,
         verify_version: bool = ...,
-        cache: Optional[AbstractCache] = ...,
         **_: Any,
     ) -> None:
         ...
@@ -499,7 +489,6 @@ class Redis(
         client_name: Optional[str] = ...,
         protocol_version: Literal[2, 3] = ...,
         verify_version: bool = ...,
-        cache: Optional[AbstractCache] = ...,
         **_: Any,
     ) -> None:
         ...
@@ -531,15 +520,9 @@ class Redis(
         client_name: Optional[str] = None,
         protocol_version: Literal[2, 3] = 2,
         verify_version: bool = True,
-        cache: Optional[AbstractCache] = None,
         **_: Any,
     ) -> None:
         """
-        .. versionadded:: 3.9.0
-           If :paramref:`cache` is provided the client will check & populate
-           the cache for read only commands and invalidate it for commands
-           that could change the key(s) in the request.
-
         .. versionchanged:: 3.5.0
            The :paramref:`verify_version` parameter now defaults to ``True``
 
@@ -559,10 +542,10 @@ class Redis(
         :param unix_socket_path: Path to the UDS which the redis server
          is listening at
         :param encoding: The codec to use to encode strings transmitted to redis
-         and decode responses with. (See :ref:`handbook/encoding:encoding/decoding`)
+         and decode responses with. (See :ref:`api_reference:encoding/decoding`)
         :param decode_responses: If ``True`` string responses from the server
          will be decoded using :paramref:`encoding` before being returned.
-         (See :ref:`handbook/encoding:encoding/decoding`)
+         (See :ref:`api_reference:encoding/decoding`)
         :param max_connections: Maximum capacity of the connection pool (Ignored if
          :paramref:`connection_pool` is not ``None``.
         :param retry_on_timeout: Whether to retry a commmand once if a :exc:`TimeoutError`
@@ -573,15 +556,11 @@ class Redis(
         :param client_name: The client name to identifiy with the redis server
         :param protocol_version: Whether to use the RESP (``2``) or RESP3 (``3``)
          protocol for parsing responses from the server (Default ``2``).
-         (See :ref:`handbook/response:parsers`)
+         (See :ref:`api_reference:parsers`)
         :param verify_version: Validate redis server version against the documented
          version introduced before executing a command and raises a
          :exc:`CommandNotSupportedError` error if the required version is higher than
          the reported server version
-        :param cache: If provided the cache will be used to avoid requests for read only
-         commands if the client has already requested the data and it hasn't been invalidated.
-         The cache is responsible for any mutations to the keys that happen outside of this client
-
         """
         super().__init__(
             host=host,
@@ -611,7 +590,6 @@ class Redis(
         )
         self._use_lua_lock: Optional[bool] = None
         self.response_callbacks: Dict[bytes, Callable[..., Any]] = {}
-        self.cache = cache
 
     @classmethod
     @overload
@@ -622,7 +600,6 @@ class Redis(
         *,
         decode_responses: Literal[False] = ...,
         protocol_version: Literal[2, 3] = ...,
-        verify_version: bool = ...,
         **kwargs: Any,
     ) -> RedisBytesT:
         ...
@@ -636,7 +613,6 @@ class Redis(
         *,
         decode_responses: Literal[True],
         protocol_version: Literal[2, 3] = ...,
-        verify_version: bool = ...,
         **kwargs: Any,
     ) -> RedisStringT:
         ...
@@ -649,7 +625,6 @@ class Redis(
         *,
         decode_responses: bool = False,
         protocol_version: Literal[2, 3] = 2,
-        verify_version: bool = True,
         **kwargs: Any,
     ) -> RedisT:
         """
@@ -683,7 +658,6 @@ class Redis(
             return cls(
                 decode_responses=True,
                 protocol_version=protocol_version,
-                verify_version=verify_version,
                 connection_pool=ConnectionPool.from_url(
                     url,
                     db=db,
@@ -696,7 +670,6 @@ class Redis(
             return cls(
                 decode_responses=False,
                 protocol_version=protocol_version,
-                verify_version=verify_version,
                 connection_pool=ConnectionPool.from_url(
                     url,
                     db=db,
@@ -706,12 +679,6 @@ class Redis(
                 ),
             )
 
-    async def initialize(self) -> Redis[AnyStr]:
-        await super().initialize()
-        if self.cache:
-            await self.cache.initialize(self)
-        return self
-
     def set_response_callback(
         self, command: StringT, callback: Callable[..., Any]
     ) -> None:
@@ -730,20 +697,10 @@ class Redis(
         **options: Optional[ValueT],
     ) -> R:
         """Executes a command and returns a parsed response"""
-        await self
-
         pool = self.connection_pool
-        connection = await pool.get_connection(command, *args)
-        if self.cache and connection.tracking_client_id != self.cache.get_client_id(
-            connection
-        ):
-            self.cache.reset()
-            await connection.update_tracking_client(
-                True, self.cache.get_client_id(connection)
-            )
+        connection = await pool.get_connection()
+
         try:
-            if self.cache and command not in self.connection_pool.READONLY_COMMANDS:
-                self.cache.reset(*KeySpec.extract_keys((command,) + args))
             await connection.send_command(command, *args)
             if custom_callback := self.response_callbacks.get(command):
                 return custom_callback(  # type: ignore
@@ -864,7 +821,9 @@ class Redis(
 
     async def transaction(
         self,
-        func: Callable[["coredis.pipeline.Pipeline[AnyStr]"], Coroutine[Any, Any, Any]],
+        func: Callable[
+            ["coredis.pipeline.Pipeline[AnyStr]"], Coroutine[Any, Any, None]
+        ],
         *watches: KeyT,
         **kwargs: Any,
     ) -> Optional[Any]:
@@ -968,7 +927,6 @@ class RedisCluster(
         protocol_version: Literal[2, 3] = 2,
         verify_version: bool = True,
         non_atomic_cross_slot: bool = False,
-        cache: Optional[AbstractCache] = None,
         **kwargs: Any,
     ):
         """
@@ -997,12 +955,12 @@ class RedisCluster(
          side the cluster if the cluster nodes move around alot.
         :param decode_responses: If ``True`` string responses from the server
          will be decoded using :paramref:`encoding` before being returned.
-         (See :ref:`handbook/encoding:encoding/decoding`)
+         (See :ref:`api_reference:encoding/decoding`)
         :param connection_pool: The connection pool instance to use. If not provided
          a new pool will be assigned to this client.
         :param protocol_version: Whether to use the RESP (``2``) or RESP3 (``3``)
          protocol for parsing responses from the server (Default ``2``).
-         (See :ref:`handbook/response:parsers`)
+         (See :ref:`api_reference:parsers`)
         :param verify_version: Validate redis server version against the documented
          version introduced before executing a command and raises a
          :exc:`CommandNotSupportedError` error if the required version is higher than
@@ -1064,7 +1022,6 @@ class RedisCluster(
             bytes, Callable[..., Any]
         ] = self.__class__.RESULT_CALLBACKS.copy()
         self.non_atomic_cross_slot = non_atomic_cross_slot
-        self.cache = cache
 
     @classmethod
     @overload
@@ -1076,7 +1033,6 @@ class RedisCluster(
         skip_full_coverage_check: bool = ...,
         decode_responses: Literal[False] = ...,
         protocol_version: Literal[2, 3] = ...,
-        verify_version: bool = ...,
         **kwargs: Any,
     ) -> RedisClusterBytesT:
         ...
@@ -1091,7 +1047,6 @@ class RedisCluster(
         skip_full_coverage_check: bool = ...,
         decode_responses: Literal[True],
         protocol_version: Literal[2, 3] = ...,
-        verify_version: bool = ...,
         **kwargs: Any,
     ) -> RedisClusterStringT:
         ...
@@ -1105,7 +1060,6 @@ class RedisCluster(
         skip_full_coverage_check: bool = False,
         decode_responses: bool = False,
         protocol_version: Literal[2, 3] = 2,
-        verify_version: bool = True,
         **kwargs: Any,
     ) -> RedisClusterT:
         """
@@ -1136,7 +1090,6 @@ class RedisCluster(
             return cls(
                 decode_responses=True,
                 protocol_version=protocol_version,
-                verify_version=verify_version,
                 connection_pool=ClusterConnectionPool.from_url(
                     url,
                     db=db,
@@ -1150,7 +1103,6 @@ class RedisCluster(
             return cls(
                 decode_responses=False,
                 protocol_version=protocol_version,
-                verify_version=verify_version,
                 connection_pool=ClusterConnectionPool.from_url(
                     url,
                     db=db,
@@ -1161,13 +1113,6 @@ class RedisCluster(
                 ),
             )
 
-    async def initialize(self) -> RedisCluster[AnyStr]:
-        await super().initialize()
-        if self.cache:
-            await self.cache.initialize(self)
-        self.refresh_table_asap = False
-        return self
-
     def __repr__(self) -> str:
         servers = list(
             {
@@ -1219,7 +1164,7 @@ class RedisCluster(
 
         if len(args) <= 0:
             raise RedisClusterException(
-                f"No way to dispatch command:{str(command)} with args: {args}"
+                f"No way to dispatch command:{command!r} with args:{args}"
                 " to Redis Cluster. Missing key."
             )
         keys: Tuple[ValueT, ...] = KeySpec.extract_keys(
@@ -1240,13 +1185,13 @@ class RedisCluster(
         slots = {self.connection_pool.nodes.keyslot(key) for key in keys}
         if not slots:
             raise RedisClusterException(
-                f"No way to dispatch command:{str(command)} with args: {args}"
+                f"No way to dispatch command:{command!r} with args:{args}"
                 " to Redis Cluster. Missing key."
             )
 
         if len(slots) != 1:
             raise RedisClusterException(
-                f"{str(command)} - all keys must map to the same key slot"
+                f"{command!r} - all keys must map to the same key slot"
             )
         return slots.pop()
 
@@ -1303,22 +1248,19 @@ class RedisCluster(
         Sends a command to a node in the cluster
         """
         if not self.connection_pool.initialized:
-            await self
+            await self.connection_pool.initialize()
 
         if not command:
             raise RedisClusterException("Unable to determine command to use")
 
         nodes = self.determine_node(command, **kwargs)
+
         if nodes:
-            try:
-                return await self.execute_command_on_nodes(
-                    nodes, command, *args, callback=callback, **kwargs
-                )
-            except ClusterDownError:
-                self.connection_pool.disconnect()
-                self.connection_pool.reset()
-                self.refresh_table_asap = True
-                raise
+            return await self.execute_command_on_nodes(
+                nodes, command, *args, callback=callback, **kwargs
+            )
+
+        # If set externally we must update it before calling any commands
 
         if self.refresh_table_asap:
             await self.connection_pool.nodes.initialize()
@@ -1363,11 +1305,6 @@ class RedisCluster(
                     await self.parse_response(r, decode=kwargs.get("decode"))
                     asking = False
 
-                if self.cache and r.tracking_client_id != self.cache.get_client_id(r):
-                    self.cache.reset()
-                    await r.update_tracking_client(True, self.cache.get_client_id(r))
-                if self.cache and command not in self.connection_pool.READONLY_COMMANDS:
-                    self.cache.reset(*KeySpec.extract_keys((command,) + args))
                 await r.send_command(command, *args)
 
                 return callback(
@@ -1425,7 +1362,7 @@ class RedisCluster(
                 key_start: int = args.index(keys[0])
                 key_end: int = args.index(keys[-1])
                 if not args[key_start : 1 + key_end] == keys:
-                    raise ClusterRoutingError(
+                    raise RuntimeError(
                         f"Unable to map {command.decode('latin-1')} by keys {keys}"
                     )
                 for node_name, node_keys in self.connection_pool.nodes.keys_to_nodes(
@@ -1436,19 +1373,9 @@ class RedisCluster(
                         *node_keys,  # type: ignore
                         *args[1 + key_end :],
                     )
-            if self.cache and command not in self.connection_pool.READONLY_COMMANDS:
-                self.cache.reset(*keys)
         for node in nodes:
             connection = self.connection_pool.get_connection_by_node(node)
-            if (
-                self.cache
-                and connection.tracking_client_id
-                != self.cache.get_client_id(connection)
-            ):
-                self.cache.reset()
-                await connection.update_tracking_client(
-                    True, self.cache.get_client_id(connection)
-                )
+
             # copy from redis-py
             try:
                 if node["name"] in node_arg_mapping:
@@ -1472,11 +1399,7 @@ class RedisCluster(
 
                 if not connection.retry_on_timeout and isinstance(e, TimeoutError):
                     raise
-                try:
-                    await connection.send_command(command, *args)
-                except ConnectionError as err:
-                    # if a retry attempt results in a connection error assume cluster error
-                    raise ClusterDownError(str(err))
+                await connection.send_command(command, *args)
                 res[node["name"]] = callback(
                     await self.parse_response(connection, decode=options.get("decode")),
                     version=self.protocol_version,
