@@ -11,8 +11,8 @@ import warnings
 from asyncio import AbstractEventLoop, StreamReader, StreamWriter
 from typing import cast
 
-from coredis._packer import Packer
-from coredis._utils import nativestr
+from coredis._utils import b, nativestr
+from coredis.constants import SYM_CRLF, SYM_DOLLAR, SYM_EMPTY, SYM_STAR
 from coredis.exceptions import (
     AuthenticationRequiredError,
     ConnectionError,
@@ -25,13 +25,12 @@ from coredis.parsers import BaseParser, DefaultParser
 from coredis.typing import (
     Awaitable,
     Callable,
-    ClassVar,
     Dict,
     List,
     Literal,
     Optional,
     ResponseType,
-    Set,
+    Tuple,
     Type,
     TypeVar,
     Union,
@@ -96,14 +95,10 @@ class RedisSSLContext:
 
 
 class BaseConnection:
-    client_id: Optional[int]
-    description: ClassVar[str] = "BaseConnection"
-    locator: ClassVar[str] = ""
+    description = "BaseConnection"
     _reader: Optional[StreamReader]
     _writer: Optional[StreamWriter]
     protocol_version: Literal[2, 3]
-    push_messages: asyncio.Queue[ResponseType]
-    tracking_client_id: Optional[int]
 
     def __init__(
         self,
@@ -140,21 +135,13 @@ class BaseConnection:
         self.server_version: Optional[str] = None
         self.loop = loop
         self.client_name = client_name
-        self.client_id = None
         # flag to show if a connection is waiting for response
         self.awaiting_response = False
         self.last_active_at = time.time()
-        self.packer = Packer(self.encoding)
-        self.push_messages: asyncio.Queue[ResponseType] = asyncio.Queue()
-        self.tracking_client_id = None
 
     def __repr__(self) -> str:
         return self.description.format(**self._description_args)
 
-    @property
-    def location(self) -> str:
-        return self.locator.format(**self._description_args)
-
     def __del__(self) -> None:
         try:
             self.disconnect()
@@ -224,22 +211,10 @@ class BaseConnection:
                 f"Failed to authenticate: username={self.username} & password={self.password}"
             )
 
-    async def update_tracking_client(
-        self, enabled: bool, client_id: Optional[int] = None
-    ) -> None:
-        if enabled and client_id is not None:
-            await self.send_command(b"CLIENT TRACKING", b"ON", b"REDIRECT", client_id)
-        else:
-            await self.send_command(b"CLIENT TRACKING", b"OFF")
-        if await self.read_response(decode=False) != b"OK":
-            raise ConnectionError("Unable to toggle client tracking")
-        self.tracking_client_id = client_id
-
     async def on_connect(self) -> None:
         self._parser.on_connect(self)
         hello_command_args: List[Union[int, str, bytes]] = [self.protocol_version]
         auth_attempted = False
-
         if self.username or self.password:
             hello_command_args.extend(
                 ["AUTH", self.username or b"default", self.password or b""]
@@ -248,39 +223,20 @@ class BaseConnection:
         try:
             await self.send_command(b"HELLO", *hello_command_args)
             hello_resp = await self.read_response(decode=False)
-            if isinstance(hello_resp, (list, dict)):
-                if self.protocol_version == 3:
-                    resp3 = cast(Dict[bytes, ValueT], hello_resp)
-                    if not resp3[b"proto"] == 3:
-                        raise ConnectionError(
-                            f"Unexpected response when negotiating protocol: [{resp3}]"
-                        )
-                    self.server_version = nativestr(resp3[b"version"])
-                    self.client_id = int(resp3[b"id"])
-                else:
-                    resp = cast(List[ValueT], hello_resp)
-                    self.server_version = nativestr(resp[3])
-                    self.client_id = int(resp[7])
-            else:
-                warnings.warn(
-                    (
-                        f"Unexpected response for `HELLO` command: {hello_resp!r}."
-                        " Unable to determine server version and set protocol version."
-                        " Assuming RESP v2"
-                    ),
-                    category=UserWarning,
-                )
-                self.server_version = None
-                self.client_id = None
-                if self.protocol_version == 3:
-                    raise ConnectionError("Unable to set RESP3 protocol version")
-        except (UnknownCommandError, AuthenticationRequiredError):
-            if self.protocol_version == 3:
-                raise ConnectionError("Unable to set RESP3 protocol version")
-            self.server_version = None
-            self.client_id = None
-            auth_attempted = False
 
+            if self.protocol_version == 3:
+                resp3 = cast(Dict[bytes, ValueT], hello_resp)
+                if not resp3[b"proto"] == 3:
+                    raise ConnectionError(
+                        f"Unexpected response when negotiating protocol: [{resp3}]"
+                    )
+                self.server_version = nativestr(resp3[b"version"])
+            else:
+                resp = cast(List[ValueT], hello_resp)
+                self.server_version = nativestr(resp[3])
+        except (UnknownCommandError, AuthenticationRequiredError):
+            self.version = None
+            auth_attempted = False
         if not auth_attempted and (self.username or self.password):
             if self.username and self.password:
                 await self.send_command(b"AUTH", self.username, self.password)
@@ -297,21 +253,16 @@ class BaseConnection:
 
         if self.client_name is not None:
             await self.send_command(b"CLIENT SETNAME", self.client_name)
+
             if await self.read_response(decode=False) != b"OK":
                 raise ConnectionError(f"Failed to set client name: {self.client_name}")
 
         self.last_active_at = time.time()
 
-    async def read_response(
-        self,
-        decode: Optional[bool] = None,
-        push_message_types: Optional[Set[bytes]] = None,
-    ) -> ResponseType:
+    async def read_response(self, decode: Optional[bool] = None) -> ResponseType:
         try:
             response = await exec_with_timeout(
-                self._parser.read_response(
-                    decode=decode, push_message_types=push_message_types
-                ),
+                self._parser.read_response(decode=decode),
                 self._stream_timeout,
             )
             self.last_active_at = time.time()
@@ -351,13 +302,23 @@ class BaseConnection:
             else:
                 raise
 
-    async def send_command(self, command: bytes, *args: ValueT) -> None:
+    async def send_command(self, *args: ValueT) -> None:
         if not self.is_connected:
             await self.connect()
-        await self.send_packed_command(self.packer.pack_command(command, *args))
+        await self.send_packed_command(self.pack_command(*args))
         self.awaiting_response = True
         self.last_active_at = time.time()
 
+    def encode(self, value: ValueT) -> bytes:
+        """Returns a bytestring representation of the value"""
+        if isinstance(value, bytes):
+            return value
+        elif isinstance(value, int):
+            return b"%d" % value
+        elif isinstance(value, float):
+            return b"%.15g" % value
+        return value.encode(self.encoding)
+
     def disconnect(self) -> None:
         """Disconnects from the Redis server"""
         self._parser.on_disconnect()
@@ -369,10 +330,62 @@ class BaseConnection:
         self._reader = None
         self._writer = None
 
+    def pack_command(self, *args: ValueT) -> List[bytes]:
+        "Pack a series of arguments into the Redis protocol"
+        output: List[bytes] = []
+        # the client might have included 1 or more literal arguments in
+        # the command name, e.g., 'CONFIG GET'. The Redis server expects these
+        # arguments to be sent separately, so split the first argument
+        # manually. All of these arguements get wrapped in the Token class
+        # to prevent them from being encoded.
+        command = b(args[0])
+        if b" " in command:
+            args = tuple(s for s in command.split()) + args[1:]
+        else:
+            args = (command,) + args[1:]
+
+        buff = SYM_EMPTY.join((SYM_STAR, b"%d" % len(args), SYM_CRLF))
+
+        for arg in map(self.encode, args):
+            # to avoid large string mallocs, chunk the command into the
+            # output list if we're sending large values
+
+            if len(buff) > 6000 or len(arg) > 6000:
+                buff = SYM_EMPTY.join((buff, SYM_DOLLAR, b"%d" % len(arg), SYM_CRLF))
+                output.append(buff)
+                output.append(arg)
+                buff = SYM_CRLF
+            else:
+                buff = SYM_EMPTY.join(
+                    (buff, SYM_DOLLAR, b"%d" % len(arg), SYM_CRLF, arg, SYM_CRLF)
+                )
+        output.append(buff)
+        return output
+
+    def pack_commands(self, commands: List[Tuple[ValueT, ...]]) -> List[bytes]:
+        "Pack multiple commands into the Redis protocol"
+        output: List[bytes] = []
+        pieces: List[bytes] = []
+        buffer_length = 0
+
+        for cmd in commands:
+            for chunk in self.pack_command(*cmd):
+                pieces.append(chunk)
+                buffer_length += len(chunk)
+
+            if buffer_length > 6000:
+                output.append(SYM_EMPTY.join(pieces))
+                buffer_length = 0
+                pieces = []
+
+        if pieces:
+            output.append(SYM_EMPTY.join(pieces))
+
+        return output
+
 
 class Connection(BaseConnection):
-    description: ClassVar[str] = "Connection<host={host},port={port},db={db}>"
-    locator: ClassVar[str] = "host={host},port={port}"
+    description = "Connection<host={host},port={port},db={db}>"
 
     def __init__(
         self,
@@ -448,15 +461,14 @@ class Connection(BaseConnection):
                         sock.setsockopt(socket.SOL_TCP, k, v)
             except (OSError, TypeError):
                 # `socket_keepalive_options` might contain invalid options
-                # causing an error. Do not leavo999e the connection open.
+                # causing an error. Do not leave the connection open.
                 writer.close()
                 raise
         await self.on_connect()
 
 
 class UnixDomainSocketConnection(BaseConnection):
-    description: ClassVar[str] = "UnixDomainSocketConnection<path={path},db={db}>"
-    locator: ClassVar[str] = "path={path}"
+    description = "UnixDomainSocketConnection<path={path},db={db}>"
 
     def __init__(
         self,
@@ -509,8 +521,7 @@ class UnixDomainSocketConnection(BaseConnection):
 
 class ClusterConnection(Connection):
     "Manages TCP communication to and from a Redis server"
-    description: ClassVar[str] = "ClusterConnection<host={host},port={port}>"
-    locator: ClassVar[str] = "host={host},port={port}"
+    description = "ClusterConnection<host={host},port={port}>"
     node: Node
 
     def __init__(
