@@ -66,16 +66,7 @@ class DataError(RedisError):
 
 
 class NoKeyError(RedisError):
-    """
-    Raised when a key provided in the command is missing
-    """
-
-
-class WrongTypeError(ResponseError):
-    """
-    Raised when an operation is performed on a key
-    containing a datatype that doesn't support the operation
-    """
+    pass
 
 
 class PubSubError(RedisError):
@@ -130,10 +121,6 @@ class ClusterCrossSlotError(ResponseError):
         self.keys = keys
 
 
-class ClusterRoutingError(RedisClusterException):
-    """Raised when keys in request can't be routed to destination nodes"""
-
-
 class ClusterDownError(ClusterError, ResponseError):
     """
     Error indicated ``CLUSTERDOWN`` error received from cluster.
@@ -275,26 +262,8 @@ class UnknownCommandError(ResponseError):
 
 
 class StreamConsumerError(RedisError):
-    """
-    Base exception for stream consumer related errors
-    """
-
-
-class StreamConsumerGroupError(StreamConsumerError):
-    """
-    Base exception for consumer group related errors
-    """
-
-
-class StreamDuplicateConsumerGroupError(StreamConsumerGroupError):
-    """
-    Raised when and attempt to create a stream consumer
-    group fails because it already exists
-    """
+    pass
 
 
 class StreamConsumerInitializationError(StreamConsumerError):
-    """
-    Raised when a stream consumer could not be initialized
-    based on the configuration provided
-    """
+    pass
