@@ -1,18 +1,29 @@
+from __future__ import annotations
+
 import asyncio
 import contextvars
+import functools
 import time as mod_time
 import uuid
 import warnings
-from typing import Optional, Union
+from types import SimpleNamespace, TracebackType
+from typing import TYPE_CHECKING, Any, cast
 
-from coredis.commands.script import Script
+from coredis._utils import nativestr
+from coredis.commands import Script
+from coredis.commands.constants import CommandName
 from coredis.connection import ClusterConnection
 from coredis.exceptions import LockError, WatchError
+from coredis.pool import ClusterConnectionPool
 from coredis.tokens import PureToken
-from coredis.utils import b, dummy, nativestr
+from coredis.typing import AnyStr, Generic, Optional, StringT, Type, Union
+
+if TYPE_CHECKING:
+    import coredis.client
+    import coredis.pipeline
 
 
-class Lock:
+class Lock(Generic[AnyStr]):
     """
     A shared, distributed Lock. Using Redis for locking allows the Lock
     to be shared across processes and/or machines.
@@ -21,17 +32,17 @@ class Lock:
     multiple clients play nicely together.
     """
 
-    local: Union[dummy, contextvars.ContextVar]
+    local: Union[SimpleNamespace, contextvars.ContextVar[Optional[StringT]]]
 
     def __init__(
         self,
-        redis,
-        name,
-        timeout=None,
-        sleep=0.1,
-        blocking=True,
-        blocking_timeout=None,
-        thread_local=True,
+        redis: coredis.client.Redis[AnyStr] | coredis.client.RedisCluster[AnyStr],
+        name: StringT,
+        timeout: Optional[float] = None,
+        sleep: float = 0.1,
+        blocking: bool = True,
+        blocking_timeout: Optional[float] = None,
+        thread_local: bool = True,
     ):
         """
         Create a new Lock instance named ``name`` using the Redis client
@@ -83,7 +94,9 @@ class Lock:
         is that these cases aren't common and as such default to using
         thread local storage.
         """
-        self.redis = redis
+        self.redis: coredis.client.Redis[AnyStr] | coredis.client.RedisCluster[
+            AnyStr
+        ] = redis
         self.name = name
         self.timeout = timeout
         self.sleep = sleep
@@ -91,23 +104,33 @@ class Lock:
         self.blocking_timeout = blocking_timeout
         self.thread_local = bool(thread_local)
         self.local = (
-            contextvars.ContextVar("token", default=None)
-            if self.thread_local
-            else dummy()
+            contextvars.ContextVar[Optional[StringT]]("token", default=None)
+            if thread_local
+            else SimpleNamespace()
         )
         if self.timeout and self.sleep > self.timeout:
             raise LockError("'sleep' must be less than 'timeout'")
 
-    async def __aenter__(self):
+    async def __aenter__(
+        self,
+    ) -> Lock[AnyStr]:
+
         # force blocking, as otherwise the user would have to check whether
         # the lock was actually acquired or not.
         await self.acquire(blocking=True)
         return self
 
-    async def __aexit__(self, exc_type, exc_value, traceback):
+    async def __aexit__(
+        self,
+        exc_type: Optional[Type[BaseException]],
+        exc_value: Optional[BaseException],
+        traceback: Optional[TracebackType],
+    ) -> None:
         await self.release()
 
-    async def acquire(self, blocking=None, blocking_timeout=None):
+    async def acquire(
+        self, blocking: Optional[bool] = None, blocking_timeout: Optional[float] = None
+    ) -> bool:
         """
         Use Redis to hold a shared, distributed lock named ``name``.
         Returns True once the lock is acquired.
@@ -117,6 +140,8 @@ class Lock:
 
         ``blocking_timeout`` specifies the maximum number of seconds to
         wait trying to acquire the lock.
+
+        :raises: :exc:`~coredis.LockError`
         """
         sleep = self.sleep
         token = uuid.uuid1().hex
@@ -129,7 +154,7 @@ class Lock:
             stop_trying_at = mod_time.time() + blocking_timeout
         while True:
             if await self.do_acquire(token):
-                self.local.set(token)  # type: ignore
+                self.local.set(token)
                 return True
             if not blocking:
                 return False
@@ -137,7 +162,7 @@ class Lock:
                 return False
             await asyncio.sleep(sleep)
 
-    async def do_acquire(self, token):
+    async def do_acquire(self, token: StringT) -> bool:
         if self.timeout:
             # convert to milliseconds
             timeout = int(self.timeout * 1000)
@@ -147,31 +172,41 @@ class Lock:
             self.name, token, condition=PureToken.NX, px=timeout
         )
 
-    async def release(self):
-        """Releases the already acquired lock"""
+    async def release(self) -> None:
+        """
+        Releases the already acquired lock
+
+        :raises: :exc:`~coredis.LockError`
+        """
         expected_token = self.local.get()
         if expected_token is None:
             raise LockError("Cannot release an unlocked lock")
         self.local.set(None)
         await self.do_release(expected_token)
 
-    async def do_release(self, expected_token):
-        name = self.name
+    async def execute_release(
+        self, token: StringT, pipe: coredis.pipeline.Pipeline[AnyStr]
+    ) -> None:
+        lock_value = await pipe.get(self.name)
+        if nativestr(lock_value) != nativestr(token):
+            raise LockError("Cannot release a lock that's no longer owned")
+        await pipe.delete([self.name])
+        return None
 
-        async def execute_release(pipe):
-            lock_value = await pipe.get(name)
-            if nativestr(lock_value) != nativestr(expected_token):
-                raise LockError("Cannot release a lock that's no longer owned")
-            await pipe.delete([name])
+    async def do_release(self, expected_token: StringT) -> None:  # noqa
+        await self.redis.transaction(
+            functools.partial(self.execute_release, token=expected_token), self.name
+        )
+        return None
 
-        await self.redis.transaction(execute_release, name)
-
-    async def extend(self, additional_time):
+    async def extend(self, additional_time: float) -> bool:
         """
         Adds more time to an already acquired lock.
 
         ``additional_time`` can be specified as an integer or a float, both
         representing the number of seconds to add.
+
+        :raises: :exc:`~coredis.LockError`
         """
         if self.local.get() is None:
             raise LockError("Cannot extend an unlocked lock")
@@ -179,13 +214,15 @@ class Lock:
             raise LockError("Cannot extend a lock with no timeout")
         return await self.do_extend(additional_time)
 
-    async def do_extend(self, additional_time):
+    async def do_extend(self, additional_time: float) -> bool:  # noqa
         pipe = await self.redis.pipeline()
         await pipe.watch(self.name)
         lock_value = await pipe.get(self.name)
         if lock_value != self.local.get():
             raise LockError("Cannot extend a lock that's no longer owned")
-        expiration = await pipe.pttl(self.name)
+        expiration = cast(
+            int, await pipe.pttl(self.name)
+        )  # pipeline returns immediate after a watch
         if expiration is None or expiration < 0:
             # Redis evicted the lock key between the previous get() and now
             # we'll handle this when we call pexpire()
@@ -204,14 +241,13 @@ class Lock:
         return True
 
 
-class LuaLock(Lock):
+class LuaLock(Lock[AnyStr]):
     """
-    A lock implementation that uses Lua scripts rather than pipelines
-    and watches.
+    A lock implementation that uses Lua scripts.
     """
 
-    lua_release: Optional[Script] = None
-    lua_extend: Optional[Script] = None
+    lua_release: Optional[Script[AnyStr]] = None
+    lua_extend: Optional[Script[AnyStr]] = None
 
     # KEYS[1] - lock name
     # ARGS[1] - token
@@ -230,33 +266,59 @@ class LuaLock(Lock):
     # ARGS[2] - additional milliseconds
     # return 1 if the locks time was extended, otherwise 0
     LUA_EXTEND_SCRIPT = """
-        local token = redis.call('get', KEYS[1])
-        if not token or token ~= ARGV[1] then
-            return 0
-        end
-        local expiration = redis.call('pttl', KEYS[1])
-        if not expiration then
-            expiration = 0
-        end
-        if expiration < 0 then
-            return 0
-        end
-        redis.call('pexpire', KEYS[1], expiration + ARGV[2])
-        return 1
+local token = redis.call('get', KEYS[1])
+if not token or token ~= ARGV[1] then
+    return 0
+end
+local expiration = redis.call('pttl', KEYS[1])
+if not expiration then
+    expiration = 0
+end
+if expiration < 0 then
+    return 0
+end
+redis.call('pexpire', KEYS[1], expiration + ARGV[2])
+return 1
     """
 
-    def __init__(self, *args, **kwargs):
-        super(LuaLock, self).__init__(*args, **kwargs)
+    def __init__(
+        self,
+        redis: coredis.client.Redis[Any] | coredis.client.RedisCluster[Any],
+        name: StringT,
+        timeout: Optional[float] = None,
+        sleep: float = 0.1,
+        blocking: bool = True,
+        blocking_timeout: Optional[float] = None,
+        thread_local: bool = True,
+    ) -> None:
+        super().__init__(
+            redis=redis,
+            name=name,
+            timeout=timeout,
+            sleep=sleep,
+            blocking=blocking,
+            blocking_timeout=blocking_timeout,
+            thread_local=thread_local,
+        )
+        self.local = (
+            contextvars.ContextVar[Optional[StringT]]("token", default=None)
+            if thread_local
+            else SimpleNamespace()
+        )
         LuaLock.register_scripts(self.redis)
+        if self.timeout and self.sleep > self.timeout:
+            raise LockError("'sleep' must be less than 'timeout'")
 
     @classmethod
-    def register_scripts(cls, redis):
+    def register_scripts(
+        cls, redis: coredis.client.Redis[AnyStr] | coredis.client.RedisCluster[AnyStr]
+    ) -> None:
         if cls.lua_release is None:
             cls.lua_release = redis.register_script(cls.LUA_RELEASE_SCRIPT)
         if cls.lua_extend is None:
             cls.lua_extend = redis.register_script(cls.LUA_EXTEND_SCRIPT)
 
-    async def do_release(self, expected_token):
+    async def do_release(self, expected_token: StringT) -> None:
         assert self.lua_release
         if not bool(
             await self.lua_release.execute(
@@ -265,13 +327,13 @@ class LuaLock(Lock):
         ):
             raise LockError("Cannot release a lock that's no longer owned")
 
-    async def do_extend(self, additional_time):
+    async def do_extend(self, additional_time: float) -> bool:
         assert self.lua_extend
         additional_time = int(additional_time * 1000)
         if not bool(
             await self.lua_extend.execute(
                 keys=[self.name],
-                args=[self.local.get(), additional_time],
+                args=[cast(AnyStr, self.local.get()), additional_time],
                 client=self.redis,
             )
         ):
@@ -279,10 +341,10 @@ class LuaLock(Lock):
         return True
 
 
-class ClusterLock(LuaLock):
+class ClusterLock(LuaLock[AnyStr]):
     """
     Cluster lock is supposed to solve lock problem in redis cluster.
-    Since high availability is provided by redis cluster using master-slave model,
+    Since high availability is provided by redis cluster using master-replica model,
     the kind of lock aims to solve the fail-over problem referred in distributed lock
     post given by redis official.
 
@@ -292,54 +354,81 @@ class ClusterLock(LuaLock):
     in a specific rule and make sure they do not migrated in cluster.
     In the worst situation, all key slots may exists in one node.
     Then the availability will be the same as one key in one node.
-    For more discussion please see:
-    https://github.com/alisaifee/coredis/issues/55
+    For more discussion please see: `<https://github.com/NoneGG/aredis/issues/55>`__
 
-    My solution is to take advantage of `READONLY` mode of slaves to ensure
-    the lock key is synced from master to N/2 + 1 of its slaves to avoid the fail-over problem.
+    The solution is to take advantage of `READONLY` mode of replicas to ensure
+    the lock key is synced from master to N/2 + 1 of its replicas to avoid the fail-over problem.
     Since it is a single-key solution, the migration problem also do no matter.
 
     Please read these article below before using this cluster lock in your app.
-    https://redis.io/topics/distlock
-    http://martin.kleppmann.com/2016/02/08/how-to-do-distributed-locking.html
-    http://antirez.com/news/101
+
+    - https://redis.io/topics/distlock
+    - http://martin.kleppmann.com/2016/02/08/how-to-do-distributed-locking.html
+    - http://antirez.com/news/101
     """
 
-    def __init__(self, *args, **kwargs):
-        super(ClusterLock, self).__init__(*args, **kwargs)
+    def __init__(
+        self,
+        redis: coredis.client.RedisCluster[AnyStr],
+        name: StringT,
+        timeout: Optional[float] = None,
+        sleep: float = 0.1,
+        blocking: bool = True,
+        blocking_timeout: Optional[float] = None,
+        thread_local: bool = True,
+    ) -> None:
+
+        super().__init__(
+            redis=redis,
+            name=name,
+            timeout=timeout,
+            sleep=sleep,
+            blocking=blocking,
+            blocking_timeout=blocking_timeout,
+            thread_local=thread_local,
+        )
+
         if not self.timeout:
             raise LockError("timeout must be provided for cluster lock")
 
-    async def check_lock_in_slaves(self, token):
+    async def check_lock_in_replicas(self, token: StringT) -> bool:  # noqa
+        assert isinstance(self.redis.connection_pool, ClusterConnectionPool)
         node_manager = self.redis.connection_pool.nodes
         slot = node_manager.keyslot(self.name)
-        master_node_id = node_manager.node_from_slot(slot)["node_id"]
-        slave_nodes = await self.redis.cluster_slaves(master_node_id)
-        count, quorum = 0, (len(slave_nodes) // 2) + 1
-        conn_kwargs = self.redis.connection_pool.connection_kwargs
-        conn_kwargs["readonly"] = True
-        for node in slave_nodes:
-            try:
-                # todo: a little bit dirty here, try to reuse Redis later
-                # todo: it may be optimized by using a new connection pool
-                conn = ClusterConnection(
-                    host=node["host"], port=node["port"], **conn_kwargs
-                )
-                await conn.send_command("get", self.name)
-                res = await conn.read_response()
-                if res == token:
-                    count += 1
-                if count >= quorum:
-                    return True
-            except Exception as exc:
-                warnings.warn(
-                    "error {} during check lock {} status in slave nodes".format(
-                        exc, self.name
+        primary_node = node_manager.node_from_slot(slot)
+        if primary_node:
+            primary_node_id = primary_node["node_id"]
+            assert primary_node_id
+            replica_nodes = await self.redis.cluster_replicas(primary_node_id)
+            count, quorum = 0, (len(replica_nodes) // 2) + 1
+            conn_kwargs = self.redis.connection_pool.connection_kwargs
+            conn_kwargs["readonly"] = True
+            for node in replica_nodes:
+                try:
+                    # todo: a little bit dirty here, try to reuse Redis later
+                    # todo: it may be optimized by using a new connection pool
+                    conn = ClusterConnection(
+                        host=node["host"],
+                        port=node["port"],
+                        **conn_kwargs,  # type: ignore
+                    )
+                    await conn.send_command(CommandName.GET, self.name)
+                    res = await conn.read_response()
+                    if nativestr(res) == nativestr(token):
+                        count += 1
+                    if count >= quorum:
+                        return True
+                except Exception as exc:
+                    warnings.warn(
+                        "error {} during check lock {} status in replica nodes".format(
+                            exc, nativestr(self.name)
+                        )
                     )
-                )
         return False
 
-    async def acquire(self, blocking=None, blocking_timeout=None):
+    async def acquire(
+        self, blocking: Optional[bool] = None, blocking_timeout: Optional[float] = None
+    ) -> bool:
         """
         Use Redis to hold a shared, distributed lock named ``name``.
         Returns True once the lock is acquired.
@@ -350,27 +439,29 @@ class ClusterLock(LuaLock):
         ``blocking_timeout`` specifies the maximum number of seconds to
         wait trying to acquire the lock. It should not be greater than
         expire time of the lock
+
+        :raises: :exc:`~coredis.LockError`
         """
         sleep = self.sleep
-        token = b(uuid.uuid1().hex)
+        token = uuid.uuid1().hex
         if blocking is None:
             blocking = self.blocking
         if blocking_timeout is None:
             blocking_timeout = self.blocking_timeout
         blocking_timeout = blocking_timeout or self.timeout
-        stop_trying_at = mod_time.time() + min(blocking_timeout, self.timeout)
+        stop_trying_at = mod_time.time() + min(blocking_timeout or 0, self.timeout or 0)
 
         while True:
             if await self.do_acquire(token):
                 lock_acquired_at = mod_time.time()
-                if await self.check_lock_in_slaves(token):
+                if await self.check_lock_in_replicas(token):
                     check_finished_at = mod_time.time()
                     # if time expends on acquiring lock is greater than given time
                     # the lock should be released manually
                     if check_finished_at > stop_trying_at:
                         await self.do_release(token)
                         return False
-                    self.local.set(token)  # type: ignore
+                    self.local.set(token)
                     # validity time is considered to be the
                     # initial validity time minus the time elapsed during check
                     await self.do_extend(lock_acquired_at - check_finished_at)
@@ -382,7 +473,7 @@ class ClusterLock(LuaLock):
                 return False
             await asyncio.sleep(sleep)
 
-    async def do_release(self, expected_token):
-        await super(ClusterLock, self).do_release(expected_token)
-        if await self.check_lock_in_slaves(expected_token):
-            raise LockError("Lock is released in master but not in slave yet")
+    async def do_release(self, expected_token: StringT) -> None:
+        await super().do_release(expected_token)
+        if await self.check_lock_in_replicas(expected_token):
+            raise LockError("Lock is released in master but not in replica yet")
