@@ -153,7 +153,6 @@ class NodeManager:
             "parser_class",
             "reader_read_size",
             "loop",
-            "protocol_version",
         )
         connection_kwargs = {
             k: v for k, v in self.connection_kwargs.items() if k in allowed_keys
@@ -242,6 +241,7 @@ class NodeManager:
                                     f" {', '.join(disagreements)}"
                                 )
 
+                self.populate_startup_nodes()
                 self.refresh_table_asap = False
 
             if self._skip_full_coverage_check:
@@ -275,7 +275,6 @@ class NodeManager:
         self.slots = tmp_slots
         self.nodes = nodes_cache
         self.reinitialize_counter = 0
-        self.populate_startup_nodes()
 
     async def increment_reinitialize_counter(self, ct: int = 1) -> None:
         for _ in range(1, ct):
@@ -298,11 +297,8 @@ class NodeManager:
 
         # at least one node should have cluster-require-full-coverage yes
         for node in nodes.values():
-            try:
-                if await self.node_require_full_coverage(node):
-                    return True
-            except ConnectionError:
-                continue
+            if await self.node_require_full_coverage(node):
+                return True
         return False
 
     def set_node_name(self, node: Node) -> None:
@@ -337,9 +333,17 @@ class NodeManager:
         """
         Do something with all startup nodes and filters out any duplicates
         """
-        self.startup_nodes.clear()
+        for item in self.startup_nodes:
+            self.set_node_name(item)
+
         for n in self.nodes.values():
-            self.startup_nodes.append(n)
+            if n not in self.startup_nodes:
+                self.startup_nodes.append(n)
+
+        # freeze it so we can set() it
+        uniq = {frozenset(node.items()) for node in self.startup_nodes}
+        # then thaw it back out into a list of dicts
+        self.startup_nodes = [dict(node) for node in uniq]  # type: ignore
 
     async def reset(self) -> None:
         """Drops all node data and start over from startup_nodes"""
