@@ -149,9 +149,7 @@ class NodeCommands:
         # send all the commands and catch connection and timeout errors.
         try:
             await connection.send_packed_command(
-                connection.packer.pack_commands(
-                    [(c.command,) + c.args for c in commands]
-                )
+                connection.pack_commands([(c.command,) + c.args for c in commands])
             )
         except (ConnectionError, TimeoutError) as e:
             for c in commands:
@@ -284,7 +282,6 @@ class PipelineImpl(AbstractRedis[AnyStr], metaclass=PipelineMeta):
         self._transaction = transaction
         self.watching = False
         self.command_stack = []
-        self.cache = None  # not implemented.
 
     async def __aenter__(self) -> "PipelineImpl[AnyStr]":
         return self
@@ -449,9 +446,7 @@ class PipelineImpl(AbstractRedis[AnyStr], metaclass=PipelineMeta):
             commands,
             [PipelineCommand(command=CommandName.EXEC, args=())],
         )
-        all_cmds = connection.packer.pack_commands(
-            [(cmd.command,) + cmd.args for cmd in cmds]
-        )
+        all_cmds = connection.pack_commands([(cmd.command,) + cmd.args for cmd in cmds])
         await connection.send_packed_command(all_cmds)
         errors: List[Tuple[int, Optional[RedisError]]] = []
 
@@ -533,7 +528,7 @@ class PipelineImpl(AbstractRedis[AnyStr], metaclass=PipelineMeta):
         raise_on_error: bool,
     ) -> Tuple[Any, ...]:
         # build up all commands into a single request to increase network perf
-        all_cmds = connection.packer.pack_commands(
+        all_cmds = connection.pack_commands(
             [(cmd.command,) + cmd.args for cmd in commands]
         )
         await connection.send_packed_command(all_cmds)
@@ -735,7 +730,6 @@ class ClusterPipelineImpl(AbstractRedis[AnyStr], metaclass=ClusterPipelineMeta):
         self.watches: Optional[Parameters[KeyT]] = watches or None
         self.watching = False
         self.explicit_transaction = False
-        self.cache = None  # not implemented.
 
     async def watch(self, *keys: KeyT) -> bool:
         raise NotImplementedError
@@ -1044,7 +1038,7 @@ class ClusterPipelineImpl(AbstractRedis[AnyStr], metaclass=ClusterPipelineMeta):
 
         if not keys:
             raise RedisClusterException(
-                f"No way to dispatch {command} to Redis Cluster. Missing key"
+                "No way to dispatch {} to Redis Cluster. Missing key".format(command)
             )
 
         slots = {self.connection_pool.nodes.keyslot(key) for key in keys}
