@@ -23,7 +23,6 @@ from coredis.exceptions import ConnectionError, RedisClusterException
 from coredis.nodemanager import Node, NodeManager
 from coredis.typing import (
     Callable,
-    ClassVar,
     Dict,
     Iterable,
     List,
@@ -67,7 +66,7 @@ _CPT = TypeVar("_CPT", bound="ConnectionPool")
 class ConnectionPool:
     """Generic connection pool"""
 
-    READONLY_COMMANDS: ClassVar[Set[bytes]] = set()
+    READONLY_COMMANDS: Set[bytes] = set()
 
     @classmethod
     def from_url(
@@ -282,10 +281,7 @@ class ConnectionPool:
                 self.reset()
 
     async def get_connection(
-        self,
-        command_name: Optional[bytes] = None,
-        *args: ValueT,
-        **kwargs: Optional[ValueT],
+        self, command_name: Optional[str] = None, *args: str, **kwargs: str
     ) -> Connection:
         """Gets a connection from the pool"""
         self.checkpid()
@@ -294,7 +290,7 @@ class ConnectionPool:
         except IndexError:
             if self._created_connections >= self.max_connections:
                 raise ConnectionError("Too many connections")
-            connection = self._make_connection(**kwargs)  # type: ignore
+            connection = self._make_connection()
         self._in_use_connections.add(connection)
 
         return connection
@@ -324,17 +320,13 @@ class ConnectionPool:
             connection.disconnect()
             self._created_connections -= 1
 
-    def _make_connection(
-        self, node: Optional[Node] = None, **options: Optional[ValueT]
-    ) -> Connection:
+    def _make_connection(self, node: Optional[Node] = None) -> Connection:
         """
         Creates a new connection
         """
 
         self._created_connections += 1
-        connection = self.connection_class(
-            **self.connection_kwargs,  # type: ignore
-        )
+        connection = self.connection_class(**self.connection_kwargs)  # type: ignore
 
         if self.max_idle_time > self.idle_check_interval > 0:
             # do not await the future
@@ -424,10 +416,7 @@ class BlockingConnectionPool(ConnectionPool):
         super().reset()
 
     async def get_connection(
-        self,
-        command_name: Optional[bytes] = None,
-        *args: ValueT,
-        **kwargs: Optional[ValueT],
+        self, command_name: Optional[str] = None, *args: str, **kwargs: str
     ) -> Connection:
         """Gets a connection from the pool"""
         self.checkpid()
@@ -496,6 +485,7 @@ class ClusterConnectionPool(ConnectionPool):
     Custom connection pool for :class:`~coredis.RedisCluster` client
     """
 
+    RedisClusterDefaultTimeout = None
     nodes: NodeManager
     connection_class: Type[ClusterConnection]
 
@@ -567,7 +557,9 @@ class ClusterConnectionPool(ConnectionPool):
         self.reset()
 
         if "stream_timeout" not in self.connection_kwargs:
-            self.connection_kwargs["stream_timeout"] = None
+            self.connection_kwargs[
+                "stream_timeout"
+            ] = ClusterConnectionPool.RedisClusterDefaultTimeout
 
     def __repr__(self) -> str:
         """
@@ -629,25 +621,22 @@ class ClusterConnectionPool(ConnectionPool):
                 self.reset()
 
     async def get_connection(
-        self,
-        command_name: Optional[bytes] = None,
-        *keys: ValueT,
-        **options: Optional[ValueT],
+        self, command_name: Optional[str] = None, *keys: str, **options: str
     ) -> Connection:
         # Only pubsub command/connection should be allowed here
 
-        if command_name != b"pubsub":
+        if command_name != "pubsub":
             raise RedisClusterException(
                 "Only 'pubsub' commands can use get_connection()"
             )
 
-        routing_key = options.pop("channel", None)
+        channel = options.pop("channel", None)
         node_type = options.pop("node_type", "master")
 
-        if not routing_key:
+        if not channel:
             return self.get_random_connection()
 
-        slot = self.nodes.keyslot(routing_key)
+        slot = self.nodes.keyslot(channel)
         if node_type == "replica":
             node = self.get_replica_node_by_slot(slot)
         else:
@@ -666,9 +655,7 @@ class ClusterConnectionPool(ConnectionPool):
 
         return connection
 
-    def _make_connection(
-        self, node: Optional[Node] = None, **options: Optional[ValueT]
-    ) -> Connection:
+    def _make_connection(self, node: Optional[Node] = None) -> Connection:
         """Creates a new connection"""
 
         assert node
