@@ -1,7 +1,5 @@
 from __future__ import annotations
 
-import asyncio
-from asyncio import StreamReader, StreamWriter
 from types import TracebackType
 
 from typing_extensions import runtime_checkable
@@ -86,18 +84,3 @@ class SupportsWatch(Protocol):  # noqa
 
     async def execute(self, raise_on_error: bool = True) -> Tuple[object, ...]:
         ...
-
-
-@runtime_checkable
-class ConnectionP(Protocol):
-    decode_responses: bool
-    encoding: str
-    push_messages: asyncio.Queue[ResponseType]
-
-    @property
-    def reader(self) -> StreamReader:
-        ...
-
-    @property
-    def writer(self) -> StreamWriter:
-        ...
