@@ -4,11 +4,7 @@ from typing import Any
 
 from coredis._utils import EncodingInsensitiveDict, nativestr
 from coredis.client import AbstractRedis
-from coredis.exceptions import (
-    ResponseError,
-    StreamConsumerInitializationError,
-    StreamDuplicateConsumerGroupError,
-)
+from coredis.exceptions import ResponseError, StreamConsumerInitializationError
 from coredis.response.types import StreamEntry
 from coredis.tokens import PureToken
 from coredis.typing import (
@@ -118,7 +114,7 @@ class Consumer(Generic[AnyStr]):
         self._initialized = True
         return self
 
-    def __await__(self) -> Generator[Any, None, Consumer[AnyStr]]:
+    def __await__(self) -> Generator[Any, None, "Consumer[AnyStr]"]:
         return self.initialize().__await__()
 
     def __aiter__(self) -> Consumer[AnyStr]:
@@ -265,26 +261,24 @@ class GroupConsumer(Consumer[AnyStr]):
             raise StreamConsumerInitializationError(
                 f"Consumer group: {self.group!r} does not exist for streams: {missing_streams}"
             )
+
         for stream in self.streams:
             if self.auto_create and not group_presence.get(stream):
-                try:
-                    if not await self.client.xgroup_create(
-                        stream, self.group, PureToken.NEW_ID, mkstream=True
-                    ):
-                        raise StreamConsumerInitializationError(
-                            f"Unable to create group {self.group!r} for stream {stream!r}"
-                        )
-                except StreamDuplicateConsumerGroupError:
-                    pass
+                if not await self.client.xgroup_create(
+                    stream, self.group, PureToken.NEW_ID, mkstream=True
+                ):
+                    raise StreamConsumerInitializationError(
+                        f"Unable to create group {self.group!r} for stream {stream!r}"
+                    )
             self.state[stream].setdefault("identifier", ">")
 
         self._initialized = True
         return self
 
-    def __await__(self) -> Generator[Any, None, GroupConsumer[AnyStr]]:
+    def __await__(self) -> Generator[Any, None, "GroupConsumer[AnyStr]"]:
         return self.initialize().__await__()
 
-    def __aiter__(self) -> GroupConsumer[AnyStr]:
+    def __aiter__(self) -> "GroupConsumer[AnyStr]":
         """
         Returns the instance of the consumer itself which can be iterated over
         """
@@ -301,6 +295,7 @@ class GroupConsumer(Consumer[AnyStr]):
 
         cur = None
         cur_stream = None
+
         for stream, buffer_entries in list(self.buffer.items()):
             if buffer_entries:
                 cur_stream, cur = stream, self.buffer[stream].pop(0)
