@@ -14,10 +14,8 @@ from typing import (
     ClassVar,
     Coroutine,
     Dict,
-    Final,
     Generator,
     Generic,
-    Hashable,
     Iterable,
     Iterator,
     List,
@@ -175,7 +173,7 @@ ResponsePrimitive: TypeAlias = Optional[Union[StringT, int, float, bool]]
 ResponseType = Union[
     ResponsePrimitive,
     List[Any],
-    MutableSet[Union[ResponsePrimitive, Tuple[ResponsePrimitive, ...]]],
+    Set[ResponsePrimitive],
     Dict[ResponsePrimitive, Any],
     "coredis.exceptions.RedisError",
 ]
@@ -192,10 +190,8 @@ __all__ = [
     "Coroutine",
     "Deque",
     "Dict",
-    "Final",
     "Generic",
     "Generator",
-    "Hashable",
     "Iterable",
     "Iterator",
     "KeyT",
